/******************************************************************************
 * Copyright (C) 2006 by                             NTRU Cryptosystems, Inc. *
 *                        ---ALL RIGHTS RESERVED---                           *
 *                                                                            *
 * This software is proprietary and confidential to NTRU Cryptosystems, Inc., *
 * is furnished under a license and may be used and copied only in accordance *
 * with the terms of such license and with the inclusion of the above         *
 * copyright notice. This software or any other copies thereof may not be     *
 * provided or otherwise made available to any other person. No title to and  *
 * ownership of the software is hereby transferred.                           *
 *                                                                            *
 * The information in this software is subject to change without notice       *
 * and should not be construed as a commitment by NTRU Cryptosystems, Inc.    *
 *                                                                            *
 ******************************************************************************/
/******************************************************************************
 * File: ntru_random.c
 *
 * Contents: Routines for returning random numbers.
 *
 *****************************************************************************/

#if !defined(NTRU_TEST)

#include <stdlib.h>
#include "crypto_platform.h"
#include "ntru_random.h"


/* ntru_get_random
 *
 * Returns a specified number of random octets.
 */

#if defined(EBATS)

void
ntru_get_random(
    uint32_t    len,            /*  in - no. of random octets requested */
    uint8_t    *out)            /* out - addr for random octets */
{
    uint8_t i;

    for (i = 0; i < len; i++)
        out[i] = (uint8_t) getchar();
}

#else

#include <time.h>
#include <string.h>

void
ntru_get_random(
    uint32_t    len,            /*  in - no. of random octets requested */
    uint8_t    *out)            /* out - addr for random octets */
{
    time_t      now;
    char       *s;
    size_t      time_len, j;
    uint8_t     i;

    now = time(NULL);
    s = ctime(&now);
    time_len = strlen(s);
    for (i = 0, j = 0; i < len; i++) {
        out[i] = s[j];
        if (++j > time_len)
            j = 0;
    }
}

#endif /* EBATS */

#endif /* !NTRU_TEST */


