#include <openssl/objects.h>
#include <openssl/md5.h>
#include <openssl/rsa.h>
#include <string.h>
#include "sizes.h"

int messagesigned(
  unsigned char *m,unsigned long long *mlen,
  const unsigned char *sm,const unsigned long long smlen,
  const unsigned char pk[PUBLICKEY_BYTES],const unsigned long long pklen
)
{
  unsigned char hash[16];
  RSA *r;

  r = RSA_new();
  if (!r) return -1;

  if (smlen < MODULUS_BYTES) goto forgery;
  *mlen = smlen - MODULUS_BYTES;
  if (*mlen > 1000000000) goto error;

  r->n = BN_new(); if (!r->n) goto error;
  if (!BN_bin2bn(pk,MODULUS_BYTES,r->n)) goto error;
  r->e = BN_new(); if (!r->n) goto error;
  if (!BN_set_word(r->e,3)) goto error;

  if (RSA_size(r) > MODULUS_BYTES) goto error;

  MD5(sm + MODULUS_BYTES,smlen - MODULUS_BYTES,hash);
  if (!RSA_verify(NID_md5,hash,16,(unsigned char *) sm,MODULUS_BYTES,r)) goto forgery;
  memcpy(m,sm + MODULUS_BYTES,smlen - MODULUS_BYTES);

  RSA_free(r);
  return 0;

  error:
  RSA_free(r);
  return -1;

  forgery:
  RSA_free(r);
  return -100;
}
