#include <openssl/objects.h>
#include <openssl/md5.h>
#include <openssl/rsa.h>
#include <string.h>
#include "sizes.h"

int verification(
  const unsigned char m[SHORTHASH_BYTES],const unsigned long long mlen,
  const unsigned char sm[SIGNATURE_BYTES],const unsigned long long smlen,
  const unsigned char pk[PUBLICKEY_BYTES],const unsigned long long pklen
)
{
  unsigned char recovered[SHORTHASH_BYTES + MODULUS_BYTES];
  int recoveredlen;
  RSA *r;

  r = RSA_new();
  if (!r) return -1;

  r->n = BN_new(); if (!r->n) goto error;
  if (!BN_bin2bn(pk,MODULUS_BYTES,r->n)) goto error;
  r->e = BN_new(); if (!r->n) goto error;
  if (!BN_set_word(r->e,3)) goto error;
  if (RSA_size(r) > MODULUS_BYTES) goto error;
  recoveredlen = RSA_public_decrypt(smlen,sm,recovered,r,RSA_PKCS1_PADDING);
  if (recoveredlen < 0) goto forgery;
  if (recoveredlen != mlen) goto forgery;
  if (memcmp(recovered,m,mlen) != 0) goto forgery;

  RSA_free(r);
  return 0;

  error:
  RSA_free(r);
  return -1;

  forgery:
  RSA_free(r);
  return -100;
}
