#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include "lib_flash.h"


unsigned char
mult2(unsigned char a, unsigned char b)
{
	int i;
	unsigned long x,P = 0x00008300; /* on pourrais meme prendre /16 */
	
	x = 0;
	for(i=0;i<32;i++)
	{
			if(a & (1<<i))
			{
				x ^= (((unsigned long) b)<<i);
			}
	}

	while(x >= 128)
	{
		if( (x^P) < x)
		{
			x ^= P;
		}
		P = P/2;
	}

	return (unsigned char) x;
}

unsigned char
inv2(unsigned char a)
{
	unsigned char b;

	if(!a)
	{
		return 0;
	}

	for(b=1;;b++)
	{
		if(mult2(a,b)==1)
		{
			return b;
		}
	}
	
}

void
build_MM(unsigned char M[256*256])
{
	int i;
	FILE *out;
	unsigned char MM[2*37*37];
	unsigned char X[37];
	unsigned char exp1[71] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
	unsigned char exp2[155] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};

	for(i=0;i<37;i++)
	{
		memset(X,0,37);
		X[i] = 1;
		exponent(&(MM[37*i]),X,exp1,71,M);
	}

	for(i=0;i<37;i++)
	{
		memset(X,0,37);
		X[i] = 1;
		exponent(&(MM[37*i+37*37]),X,exp2,155,M);
	}

	out = fopen("MM2738.flash","wb");
	fwrite(MM,1,2*37*37,out);
	fclose(out);
}

void
build_M(void)
{
	FILE *out;
	int i,j;
	unsigned char M[256*256];

	for(i=0;i<128;i++)
	{
		for(j=0;j<128;j++)
		{
			M[i*256+j] = mult2((unsigned char) i,(unsigned char) j);
		}
	}

	out = fopen("M256.flash","wb");
	fwrite(M,1,256*256,out);
	fclose(out);
}

void
build_I(void)
{
	FILE *out;
	int i;
	unsigned char I[256];

	for(i=0;i<128;i++)
	{
		I[i] = inv2((unsigned char) i);
	}

	out = fopen("I256.flash","wb");
	fwrite(I,1,256,out);
	fclose(out);
}

void init_table_flash(unsigned char M[256*256], unsigned char I[256], unsigned char M1[37][37], unsigned char M2[37][37])
{
	FILE *in;
	in = fopen("M256.flash","rb");
	if(in)
	{
		fread(M,1,256*256,in);	
		fclose(in);
	}
	else
	{
/* 		printf("\n Generation de la table de multiplication sur le corp K"); */
		build_M();
		in = fopen("M256.flash","rb");
		if(in)
		{
			fread(M,1,256*256,in);	
			fclose(in);
		}
		else
		{
			printf("\n Erreur de creation de la table M");
			exit(0);
		}
	}

	in = fopen("I256.flash","rb");
	if(in)
	{
		fread(I,1,256,in);	
		fclose(in);
	}
	else
	{
/* 		printf("\n Generation de la table d'inversion sur le corp K"); */
		build_I();
		in = fopen("I256.flash","rb");
		if(in)
		{
			fread(I,1,256,in);	
			fclose(in);
		}
		else
		{
			printf("\n Erreur de creation de la table I");
			exit(0);
		}
	}

	if(!M1 || !M2)
	{
		return;
	}


	in = fopen("MM2738.flash","rb");
	if(in)
	{
		int i,j;
		unsigned char MM[37*37];

		fread(MM,1,37*37,in);
		for(i=0;i<37;i++)
		{
			for(j=0;j<37;j++)
			{
				M1[i][j] = MM[i*37+j];
			}
		}

		fread(MM,1,37*37,in);
		for(i=0;i<37;i++)
		{
			for(j=0;j<37;j++)
			{
				M2[i][j] = MM[i*37+j];
			}
		}
		fclose(in);
	}
	else
	{
/* 		printf("\n Generation de la table d'exponentiation sur le corp K"); */
		build_MM(M);
		in = fopen("MM2738.flash","rb");
		if(in)
		{
			int i,j;
			unsigned char MM[37*37];
	
			fread(MM,1,37*37,in);
			for(i=0;i<37;i++)
			{
				for(j=0;j<37;j++)
				{
					M1[i][j] = MM[i*37+j];
				}
			}

			fread(MM,1,37*37,in);
			for(i=0;i<37;i++)
			{
				for(j=0;j<37;j++)
				{
					M2[i][j] = MM[i*37+j];
				}
			}
			fclose(in);
		}
		else
		{
			printf("\n Erreur de creation de la table M2738");
			exit(0);
		}
	}
}

